@@ -4,6 +4,7 @@ use std::io::Error;
 use std::os::unix::fs::{FileExt, FileTypeExt};
 use std::os::unix::io::AsRawFd;
 use std::process::exit;
+use std::time::{Duration, Instant};
 use std::vec::Vec;
 
 use clap::Parser;
@@ -29,6 +30,10 @@ ioctl_read_bad!(blksectget, request_code_none!(0x12, 103), c_ushort);
 ioctl_read_bad!(blksszget, request_code_none!(0x12, 104), c_int);
 ioctl_read!(blkgetsize64, 0x12, 114, u64);
 
+fn rate(size: u64, duration: Duration) -> String {
+	bytesize::to_string((size as u128 * 1000 / duration.as_millis().max(1)) as u64, true)
+}
+
 fn main() -> std::io::Result<()> {
 	let opt = Opt::parse();
 
@@ -93,10 +98,17 @@ fn main() -> std::io::Result<()> {
 
 	eprintln!();
 
+	let start = Instant::now();
+	let mut last = start;
+
 	loop {
-		if verify == 0 {
-			eprintln!("\x1bM\x1b[K{:>3} %  {:>11} / {}  ({} corrupt sectors)", offset * 100 / size,
-			          bytesize::to_string(offset, true), bytesize::to_string(size, true), errors);
+		let now = Instant::now();
+
+		if verify == 0 && now.duration_since(last) > Duration::from_millis(50) {
+			eprintln!("\x1bM\x1b[K{:>3} %   {:>9} / {}   {:>9} / s   {} corrupt sectors", offset * 100 / size,
+			          bytesize::to_string(offset, true), bytesize::to_string(size, true),
+			          rate(offset, last.duration_since(now)), errors);
+			last = now;
 		}
 
 		match dev.read_at(if verify == 0 { &mut buffer } else { &mut buffer[0..ssz] }, offset) {
